﻿local _, addon = ...

local AVOIDANCE = "Avoidance"
local AVOIDANCE_BLOCK = "Avoidance + block"

local BLOCK_VALUE_TOOLTIP = CR_BLOCK_TOOLTIP:match("\n(.+)")

-- table of available stats
addon.statsDB = {
	GENERAL = {
		{value = "HEALTH",			label = HEALTH},
		{value = "DRUIDMANA",		label = MANA},
		{value = "POWER"},
		{value = "MASTERY",			label = STAT_MASTERY},
		{value = "ITEMLEVEL",		label = STAT_AVERAGE_ITEM_LEVEL},
		{value = "MOVESPEED",		label = STAT_MOVEMENT_SPEED},
	},

	ATTRIBUTES = {
		{value = "STRENGTH",	label = SPELL_STAT1_NAME},
		{value = "AGILITY",		label = SPELL_STAT2_NAME},
		{value = "STAMINA",		label = SPELL_STAT3_NAME},
		{value = "INTELLECT",	label = SPELL_STAT4_NAME},
		{value = "SPIRIT",		label = SPELL_STAT5_NAME},
	},
	
	MELEE = {
		{value = "MELEE_DAMAGE",		label = DAMAGE},
		{value = "MELEE_DPS",			label = STAT_DPS_SHORT},
		{value = "MELEE_AP",			label = STAT_ATTACK_POWER},
		{value = "MELEE_ATTACKSPEED",	label = WEAPON_SPEED},
		{value = "HASTE",				label = STAT_HASTE},
		{value = "ENERGY_REGEN",		label = STAT_ENERGY_REGEN},
		{value = "RUNE_REGEN",			label = STAT_RUNE_REGEN},
		{value = "HITCHANCE",			label = STAT_HIT_CHANCE},
		{value = "CRITCHANCE",			label = MELEE_CRIT_CHANCE},
		{value = "EXPERTISE",			label = STAT_EXPERTISE},
	},
	
	RANGED = {
		{value = "RANGED_DAMAGE",		label = DAMAGE},
		{value = "RANGED_DPS",			label = STAT_DPS_SHORT},
		{value = "RANGED_AP",			label = STAT_ATTACK_POWER},
		{value = "RANGED_ATTACKSPEED",	label = WEAPON_SPEED},
		{value = "RANGED_HASTE",		label = STAT_HASTE},
		{value = "FOCUS_REGEN",			label = STAT_FOCUS_REGEN},
		{value = "RANGED_HITCHANCE",	label = STAT_HIT_CHANCE},
		{value = "RANGED_CRITCHANCE",	label = RANGED_CRIT_CHANCE},
		{value = "EXPERTISE",			label = STAT_EXPERTISE},
	},
	
	SPELL = {
		{value = "SPELLDAMAGE",			label = STAT_SPELLDAMAGE},
		{value = "SPELLHEALING",		label = STAT_SPELLHEALING},
		{value = "SPELL_HASTE",			label = STAT_HASTE},
		{value = "SPELL_HITCHANCE",		label = STAT_HIT_CHANCE},
		{value = "MANAREGEN",			label = MANA_REGEN},
		{value = "COMBATMANAREGEN",		label = MANA_REGEN_COMBAT},
		{value = "SPELLCRIT",			label = SPELL_CRIT_CHANCE},
	},
	
	DEFENSE = {
		{value = "ARMOR",					label = ARMOR},
		{value = "DODGE",					label = STAT_DODGE},
		{value = "PARRY",					label = STAT_PARRY},
		{value = "BLOCK",					label = STAT_BLOCK},
		{value = "BLOCKVALUE",				label = ITEM_MOD_BLOCK_VALUE_SHORT},
		{value = "RESILIENCE_REDUCTION",	label = STAT_RESILIENCE},	-- resilience
		{value = "PVP_POWER",				label = STAT_PVP_POWER},	-- resilience
		{value = "AVOIDANCE",				label = AVOIDANCE},			-- avoidance (dodge + parry)
		{value = "AVOIDANCE_BLOCK",			label = AVOIDANCE_BLOCK},	-- avoidance + block
	},
	
	-- RESISTANCE = {},
}


---------------------------------------
-- these are functions that were made
-- for "custom made" stat displays
---------------------------------------

PAPERDOLL_STATINFO["BLOCKVALUE"] = {
	updateFunc = function(statFrame, unit)
		local blockValue = GetShieldBlock()
		PaperDollFrame_SetLabelAndText(statFrame, ITEM_MOD_BLOCK_VALUE_SHORT, blockValue, 1)
		statFrame.tooltip = HIGHLIGHT_FONT_COLOR_CODE..format("%s %d%%", ITEM_MOD_BLOCK_VALUE_SHORT, blockValue)..FONT_COLOR_CODE_CLOSE
		statFrame.tooltip2 = BLOCK_VALUE_TOOLTIP:format(blockValue)
		statFrame:Show()
	end
}

-- this is your combined dodge and parry chance
local function avoidance_OnEnter(self)
	local dodge, parry = GetDodgeChance(), GetParryChance()
	local avoidance = dodge + parry
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(HIGHLIGHT_FONT_COLOR_CODE..format("Avoidance %.2f%%", avoidance)..FONT_COLOR_CODE_CLOSE)
	GameTooltip:AddDoubleLine(DODGE_CHANCE, format("%.2f%%", dodge))
	GameTooltip:AddDoubleLine(PARRY_CHANCE, format("%.2f%%", parry))
	
	GameTooltip:Show()
end

PAPERDOLL_STATINFO["AVOIDANCE"] = {
	updateFunc = function(statFrame, unit)
		local avoidance = GetDodgeChance() + GetParryChance()
		PaperDollFrame_SetLabelAndText(statFrame, AVOIDANCE, avoidance, 1)
		statFrame:SetScript("OnEnter", avoidance_OnEnter)
		statFrame:Show()
	end
}

local function avoidanceBlock_OnEnter(self)
	local dodge, parry, block = GetDodgeChance(), GetParryChance(), GetBlockChance()
	local avoidance = dodge + parry + block
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(HIGHLIGHT_FONT_COLOR_CODE..format("Avoidance %.2f%%", avoidance)..FONT_COLOR_CODE_CLOSE)
	GameTooltip:AddDoubleLine(DODGE_CHANCE, format("%.2f%%", dodge))
	GameTooltip:AddDoubleLine(PARRY_CHANCE, format("%.2f%%", parry))
	GameTooltip:AddDoubleLine(BLOCK_CHANCE, format("%.2f%%", block))
	GameTooltip:Show()
end

-- same as above but including block chance
PAPERDOLL_STATINFO["AVOIDANCE_BLOCK"] = {
	updateFunc = function(statFrame)
		local avoidance = GetDodgeChance() + GetBlockChance() + GetParryChance()
		PaperDollFrame_SetLabelAndText(statFrame, AVOIDANCE_BLOCK, avoidance, 1)
		statFrame:SetScript("OnEnter", avoidanceBlock_OnEnter)
		statFrame:Show()
	end
}

local missRates = {
	{
		value = "MISS_CHANCE_SPECIAL",
		label = MISS_CHANCE,
		func = GetMeleeMissChance,
		onEnter = MeleeHitChance_OnEnter,
		category = "MELEE",
		isSpecial = true,
	},
	{
		value = "MISS_CHANCE",
		label = MISS_CHANCE.." (*)",
		func = GetMeleeMissChance,
		onEnter = MeleeHitChance_OnEnter,
		category = "MELEE",
	},
	{
		value = "RANGED_MISS_CHANCE",
		label = MISS_CHANCE,
		func = GetRangedMissChance,
		onEnter = RangedHitChance_OnEnter,
		category = "RANGED",
	},
	{
		value = "SPELL_MISS_CHANCE",
		label = MISS_CHANCE,
		func = GetSpellMissChance,
		onEnter = SpellHitChance_OnEnter,
		category = "SPELL",
	},
}

for i, v in ipairs(missRates) do
	local label = format("%s (%s)", v.label, BOSS)
	PAPERDOLL_STATINFO[v.value] = {
		updateFunc = function(statFrame)
			PaperDollFrame_SetLabelAndText(statFrame, label, v.func(3, isSpecial), 1)
			statFrame:SetScript("OnEnter", v.onEnter)
			statFrame:Show()
		end
	}
	tinsert(addon.statsDB[v.category], {value = v.value, label = label})
end

local enemyAvoidRates = {
	{value = "ENEMY_DODGE_CHANCE", label = STAT_DODGE, func = GetEnemyDodgeChance},
	{value = "ENEMY_PARRY_CHANCE", label = STAT_PARRY, func = GetEnemyParryChance},
}

for i, v in ipairs(enemyAvoidRates) do
	local label = format("%s %s (%s)", ENEMY, v.label, BOSS)
	PAPERDOLL_STATINFO[v.value] = {
		updateFunc = function(statFrame)
			PaperDollFrame_SetLabelAndText(statFrame, label, v.func(3), 1)
			statFrame:SetScript("OnEnter", Expertise_OnEnter)
			statFrame:Show()
		end
	}
	tinsert(addon.statsDB["MELEE"], {value = v.value, label = label})
	tinsert(addon.statsDB["RANGED"], {value = v.value, label = label})
end